;**********************************************************************
;                                                                     *
;    Filename:	    x10lib.asm					      *
;    Date:          10/15/02                                          *
;    File Version:   1.00                                             *
;                                                                     *
;    Author:        Jon Burroughs                                     *
;    Company:       Microchip Technology                              *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                    Software License Agreement                       *
;                                                                     *
; The software supplied herewith by Microchip Technology              *
; Incorporated (the "Company") for its PICmicro Microcontroller is   *
; intended and supplied to you, the Companys customer, for use       *
; solely and exclusively on Microchip PICmicro Microcontroller        *
; products. The software is owned by the Company and/or its           *
; supplier, and is protected under applicable copyright laws. All     *
; rights are reserved. Any use in violation of the foregoing          *
; restrictions may subject the user to criminal sanctions under       *
; applicable laws, as well as to civil liability for the breach of    *
; the terms and conditions of this license.                           *
;					                              *
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   *
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   *
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 	      *
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   *
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 	      *
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.	              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:  x10lib.inc                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:   This file provides a basic libarary of function calls   *
;	that can be used to transmit and receive commands using the   * 
;	PIC Home Automation Controller.	  		              *
;**********************************************************************

	list      p=16F877	      ; list directive to define processor
        errorlevel -302               ; suppress "register not in bank0, check page bits" message.

	#define	This_Is_X10
	#include x10lib.inc		;X10 constant definitions


;**********************************************************************
;   Variables                                                         *
;**********************************************************************
grp0	udata

X10Flags	res	1
NumTxBits	res	1

TxFunction	res	1	;transmit function code
TxHouse		res	1	;transmit house code (0-15 == A-P)
TxUnit		res	1	;transmit unit code (0-15 == 1-16)
EncodeValue	res	1	;index (0-15) to EncodeTable
DecodeValue	res	1	;X-10 house or unit value to be decoded into value(0-15)

RxFunction	res	1	;received function code
RxHouse		res	1	;received house code (0-15 == A-P)
RxUnit		res	1	;received unit code (0-15 == 1-16)

RxHouse1	res	1	;temp for house code before decode
RxHouse2	res	1	;temp for house code before decode
RxKey1		res	1	;temp for function or unit code before suffix is received.
RxKey2		res	1	;

RxData0		res	1	;receive shift registers
RxData1		res	1
RxData2		res	1
RxData3		res	1

TxData0		res	1	;transmit shift registers
TxData1		res	1
TxData2		res	1
TxData3		res	1

Count		res	1	;Counter for shifting
Dimmer		res	1


;**********************************************************************
;   Global Declarations		                                      *
;**********************************************************************
	global	ZeroCrossing, Wait6ZeroX, InitX10Tx, DecodeHouseOrUnit, EndX10Envelope, InitX10Variables	;global functions
	global	X10Flags, TxHouse, TxUnit, TxFunction, RxHouse, RxUnit, RxFunction, RxKey2, RxHouse2	;global variables


PROG1	code
;****************************************************************************************************
;X10 routines
;  some internal routines have been implemented as macros, to minimize use of the stack
;****************************************************************************************************		
;----------------------------------------------------------------------
;InitX10Variables
;----------------------------------------------------------------------
InitX10Variables
	banksel	RxData0
	clrf	RxData0
	clrf	RxData1
	clrf	RxData2
	clrf	RxData3
	clrf	RxHouse1
	clrf	RxHouse2
	clrf	RxKey1
	clrf	RxKey2
	clrf	RxUnit
	clrf	RxFunction
	clrf	TxData0
	clrf	TxData1
	clrf	TxData2
	clrf	TxData3
	clrf	TxUnit
	clrf	TxHouse
	clrf	TxFunction
	clrf	X10Flags
	return

;----------------------------------------------------------------------
; TransmitX10Bit
;	Called at every zero crossing.
;		If TxBit=1, enable 120KHz output for 1ms
;		otherwise, do nothing.
;----------------------------------------------------------------------
TransmitX10Bit	macro
	pagesel	Tx0
	btfss	TxBit		;Is TxBit a one?	
	goto	Tx0		;   No, send a zero (by doing nothing).
Tx1
	banksel	TRISC
	bcf	XOUT		;   Yes, enable a 120KHz output.
	banksel	TMR0		
	movlw	0xE2		;   Set TMR0 up for 1ms delay to time duration of 120KHz burst.
	movwf	TMR0
	bcf	INTCON, T0IF	;   Clear the Timer0 interrupt flag.
	bsf	INTCON, T0IE	;   Enable Timer0 interrupt.
Tx0
	bcf	RxBit		;Clear RxBit.
	btfsc	TxBit		;Is the TxBit set?
	bsf	RxBit		;  Yes, then set RxBit. (This assures reception of own transmission)
	bcf	TxBit		;Clear the TxBit

	endm

;----------------------------------------------------------------------
; ReceiveX10Bit
;	Called at every zero crossing.
;		If 120KHz carrier is present (XIN=1), set the RxBit
;		otherwise, clear the the RxBit.
;----------------------------------------------------------------------
ReceiveX10Bit	macro
	btfss	X10TxFlag	;If X10 transmit flag is not set,
	bcf	RxBit		;  Clear RxBit.
	btfsc	XIN		;Is 120kHz signal present?
	bsf	RxBit		;  Yes, then set RxBit.

	endm

;----------------------------------------------------------------------
;ToggleInterruptEdge
;	Toggles INTEDG bit so that interrupts are generated
;	 on both rising and falling zero-crossings on RB0/INT pin.
;----------------------------------------------------------------------
ToggleInterruptEdge	macro
	banksel	OPTION_REG
	pagesel	IntFallingEdge
	btfsc	OPTION_REG, INTEDG	;If interrupt was on rising edge
	goto	IntFallingEdge		;	set next interrupt for falling edge
	bsf	OPTION_REG, INTEDG	;	otherwise, set next interrupt for rising edge
	pagesel	EndToggleInterruptEdge
	goto	EndToggleInterruptEdge		
IntFallingEdge
	bcf	OPTION_REG, INTEDG	;	set next interrupt for falling edge
EndToggleInterruptEdge
	endm


;----------------------------------------------------------------------
; RotateTxData
;  	Rotates data in TxData register to left by number in Wreg
;----------------------------------------------------------------------
RotateTxData	macro
	movwf	Count
	bcf	STATUS, C	;Clear the carry bit
	btfsc	TxData3, 7	;Is MSb of TxData3 set?
	bsf	STATUS, C	;  Yes, set the carry bit
;RotateLoop
	rlf	TxData0, f
	rlf	TxData1, f
	rlf	TxData2, f
	rlf	TxData3, f
	decfsz	Count, f
	goto	$-5		;loop back to beginning of RotateLoop
	endm

;----------------------------------------------------------------------
; X10Tx
;
;	Transmit 22 bits of data out of TXData registers
;	Rotate by 10 to reposition TXData
;	Transmit 28 bits of data out of TXData registers
;
;----------------------------------------------------------------------	
X10Tx	macro

	bcf	TxBit		;Clear the TxBit
	btfsc	TxData3, 7	;Is MSb of TxData3 set?
	bsf	TxBit		;  Yes, set TxBit
	
	bcf	STATUS, C	;Clear the carry bit
	btfsc	TxBit		;Is TxBit set? 
	bsf	STATUS, C	;  Yes, set the carry bit.

	rlf	TxData0, F	;Rotate values in the TxData registers once to the left
	rlf	TxData1, F
	rlf	TxData2, F
	rlf	TxData3, F
	
	pagesel	EndX10Tx
	decfsz	NumTxBits, F	;Have all the bits been sent?
	goto	EndX10Tx	;  No, go to end of routine
	pagesel	Init2ndTx
	btfss	SecondTxFlag	;  Yes, was this the second transmission?
	goto	Init2ndTx	;	No, set up for second transmission 
	bsf	LED1		;	Yes, Turn off LED1
	bcf	X10TxFlag	;	Yes, transmit is done
	pagesel	EndX10Tx
	goto	EndX10Tx

Init2ndTx				
	bsf	SecondTxFlag	;Set the flag to indicate second transmission
	movlw	.10		
	RotateTxData		;Reposition data at left end of buffer
	movlw	.28		;Send 22 bits again
	movwf	NumTxBits			

EndX10Tx
	endm

;----------------------------------------------------------------------
; X10Rx
;----------------------------------------------------------------------	
X10Rx	macro
	bcf	STATUS, C	;Clear the carry bit.
	btfsc	RxBit		;Was the 120kHz signal present when ReceiveX10Bit was called?
	bsf	STATUS, C	;   Yes, set the carry bit.
	rlf	RxData0, F		;Rotate the received data into the RxData registers
	rlf	RxData1, F		
	rlf	RxData2, F
	rlf	RxData3, F
 	
	movf	RxData3, W	;Place RxData3 in W
	andlw	b'00001111'	;Mask off the 4 MSB's
	xorlw	b'00001110'	;Check for the Start Code '1110'
	pagesel	EndX10Rx
	btfss	STATUS, Z	;Is the start code present?
	goto	EndX10Rx	;   No, go to end of routine

;---When Start Code is detected in RxData3 [xxxx1110] perform the below operations:
	movf	RxHouse1, W	;transfer contents of RxHouse1 to RxHouse2
	movwf	RxHouse2
	movf	RxKey1, W	;transfer contents of RxKey1 to RxKey2
	movwf	RxKey2

	movf	RxData2, W	;Store RxData2 in RxHouse1
	movwf	RxHouse1
	movf	RxData1, W	;Store RxData1 in RxKey1
	movwf	RxKey1

	movf	RxHouse1, W
	xorwf	RxHouse2, W
	pagesel	EndX10Rx
	btfss	STATUS, Z	;Do last two received house codes match?
	goto	EndX10Rx	;  No, go to end of routine

	movf	RxKey1, W
	xorwf	RxKey2, W
	pagesel	EndX10Rx
	btfss	STATUS, Z	;Do last two received key codes match?
	goto	EndX10Rx	;  No, go to end of routine

	movf	RxData0, W	;Place RxData0 in W
	andlw	b'11000000'	;Mask off the 6 LSB's
	xorlw	b'01000000'	;Check for the Unit address suffix '01'
	pagesel	AddressRcvd
	btfsc	STATUS, Z	;Is the Unit address suffix present?
	goto	AddressRcvd	;  Yes, then address has been received.
				;  No, then check for Function suffix.
	movf	RxData0, W	;Place RxData0 in W
	andlw	b'11000000'	;Mask off the 6 LSB's
	xorlw	b'10000000'	;Check for the Function suffix '01'
	pagesel	CommandRcvd
	btfsc	STATUS, Z	;Is the Function suffix present?
	goto	CommandRcvd	;  Yes, then command has been received.
	pagesel	EndX10Rx	
	goto	EndX10Rx	;  No, then neither command or address has been properly received.

AddressRcvd
;	movf	RxKey2, W	;Transfer value in RxKey2 to RxUnit
;	movwf	RxUnit
	bcf	RxCommandFlag	;Clear command received flag since address was received
	bsf	X10RxFlag
	pagesel	EndX10Rx
	goto	EndX10Rx

CommandRcvd
;	movf	RxKey2, W	;Transfer value in RxKey2 to RxUnit
;	movwf	RxFunction
	bsf	RxCommandFlag	;Set command received flag
	bsf	X10RxFlag
;	pagesel	EndX10Rx
;	goto	EndX10Rx

EndX10Rx
	endm

;----------------------------------------------------------------------
;ZeroCrossing
;----------------------------------------------------------------------
ZeroCrossing

	TransmitX10Bit		;Transmit value of TxBit

	ReceiveX10Bit		;Update value of RxBit

	pagesel ZeroXReceive
	btfss	X10TxFlag	;If X10 transmit flag is set,
	goto	ZeroXReceive
	X10Tx			;   execute X10Tx macro

ZeroXReceive		
	X10Rx			;   execute X10Rx macro

	ToggleInterruptEdge	;set rising or falling edge for next zero-crossing
	bcf	INTCON, INTF	;clear the RB0/INT flag

	return	

;----------------------------------------------------------------------
; EndX10Envelope
;----------------------------------------------------------------------
EndX10Envelope
	banksel	TRISC
	bsf	XOUT		;disable 120kHz output
	bcf	INTCON, T0IE	;disable Timer0 interrupt
	bcf	INTCON, T0IF	;clear the Timer0 interrupt flag
	return

;----------------------------------------------------------------------
; Wait6ZeroX
;----------------------------------------------------------------------
Wait6ZeroX
	decfsz	Count, F
	goto	$+2
	bcf	WaitFlag
	return

;----------------------------------------------------------------------
; InitX10Tx
;----------------------------------------------------------------------	
InitX10Tx
	movlw	b'00001110'	;load TxData3 with start code
	movwf	TxData3

	banksel	TxHouse
	movf	TxHouse, W	;copy TxHouse into EncodeValue
	banksel EncodeValue	
	movwf	EncodeValue	
	pagesel	EncodeTable	
	call	EncodeTable	;get corresponding X-10 code for house address
	movwf	TxData2		;load encoded house address into TxData2

	pagesel	TxCommand	
	btfsc	TxCommandFlag	;Is the transmit command flag set?
	goto	TxCommand	;  Yes, then prepare to transmit command
				;  No, then prepare to transmit an address

TxAddress			;If transmitting an address
	banksel	TxUnit
	movf	TxUnit, W	;copy TxUnit into EncodeValue
	banksel EncodeValue	
	movwf	EncodeValue	
	pagesel	EncodeTable	
	call	EncodeTable	;get corresponding X-10 code for unit address
	movwf	TxData1		;load encoded unit address into TxData1
	movlw	b'01000000'	;load suffix for unit address into TxData0
	movwf	TxData0
	pagesel EndInitX10Tx
	goto	EndInitX10Tx


TxCommand			;If transmitting a command	
	banksel	TxFunction
	movf	TxFunction,W	;load function code into TxData1
	movwf	TxData1
	movlw	b'10000000'	;load suffix for function code into TxData0
	movwf	TxData0

EndInitX10Tx
	;Rotate the TxData registers 4 times to the left to prepare for transmission
	movlw	.4
	RotateTxData

	movlw	.22		;set number of bits to transmit
	movwf	NumTxBits
	bcf	SecondTxFlag	;clear the 2ndTxFlag
	bcf	LED1		;turn on LED1

	return

;****************************************************************************************************
;X-10 Table
;****************************************************************************************************	
;----------------------------------------------------------------------
;EncodeTable
;	Takes EncodeValue and returns corresponding X-10 code 
;	in true compliment format (0=01, 1=10)
;
;	Used to return value for H1 H2 H3 H4 based on housecode 0-15 ('A'-'P')
;	or value of D1 D2 D4 D16 based on unit code 0-15 ('1'-'16')
;----------------------------------------------------------------------
EncodeTable
	movlw	HIGH(EncodeTable)	;***pre-load PCLATH***
	movwf	PCLATH
	movf	EncodeValue, W
	addwf	PCL, F 		;HOUSE	UNIT
	retlw	b'01101001'	; 'A'	 1	b'0110'		
	retlw	b'10101001'	; 'B'	 2	b'1110'		
	retlw	b'01011001'	; 'C'	 3	b'0010'		
	retlw	b'10011001'	; 'D'	 4	b'1010'		
	retlw	b'01010110'	; 'E'	 5	b'0001'		
	retlw	b'10010110'	; 'F'	 6	b'1001'	
	retlw	b'01100110'	; 'G'	 7	b'0101'		
	retlw	b'10100110'	; 'H'	 8	b'1101'		
	retlw	b'01101010'	; 'I'	 9	b'0111'		
	retlw	b'10101010'	; 'J'	10	b'1111'		
	retlw	b'01011010'	; 'K'	11	b'0011'		
	retlw	b'10011010'	; 'L'	12	b'1011'		
 	retlw	b'01010101'	; 'M'	13	b'0000'		
	retlw	b'10010101'	; 'N'	14	b'1000'		
	retlw	b'01100101'	; 'O'	15	b'0100'		
	retlw	b'10100101'	; 'P'	16	b'1100'	

;----------------------------------------------------------------------
;DecodeHouseOrUnit
;	Converts true compliment format X-10 House or Unit code in WREG
;       to value (0-15).
;
;	Works forwards through EncodeTable, comparing value to decode
;	against encoded values.  When there is a match, returns the index to
;	EncodeTable that produced the match (a value from 0-15).  
;	If no values match, sets the decoded value to FF.
;
;----------------------------------------------------------------------
DecodeHouseOrUnit
	movwf	DecodeValue	;place value in WREG into DecodeValue
	clrf	EncodeValue
DecodeLoop
	pagesel	EncodeTable	
	call	EncodeTable
	xorwf	DecodeValue, W	;Is decode value same as value from EncodeTable?
	pagesel	EndDecode
	btfsc	STATUS, Z 		
	goto	EndDecode	;  Yes, return with value in EncodeValue
	pagesel	DecodeLoop
	incf	EncodeValue, F 	;  No, increment EncodeValue
	movlw	.16		;
	xorwf	EncodeValue, W	;Is EncodeValue past max index?
	btfss	STATUS, Z	; 		
	goto	DecodeLoop	;  No, repeat loop.
	movlw	0xFF		;  Yes, set EncodeValue to FF
	movwf	EncodeValue	
		
EndDecode
	movf	EncodeValue, W
	return	

;***********************
;END OF PROGRAM
;***********************
	end			;directive 'end of program'  
