;**********************************************************************
;                                                                     *
;    Filename:	    lcd.asm				     	      *
;    Date:          10/15/02                                          *
;    File Version:   1.00                                             *
;                                                                     *
;    Author:        Ross Fossler                                      *
;    Modified by:   Jon Burroughs				      *
;                                                                     *
;    Company:       Microchip Technology                              *
;                                                                     *
;**********************************************************************
;                    Software License Agreement                       *
;                                                                     *
; The software supplied herewith by Microchip Technology              *
; Incorporated (the "Company") for its PICmicro Microcontroller is   *
; intended and supplied to you, the Companys customer, for use       *
; solely and exclusively on Microchip PICmicro Microcontroller        *
; products. The software is owned by the Company and/or its           *
; supplier, and is protected under applicable copyright laws. All     *
; rights are reserved. Any use in violation of the foregoing          *
; restrictions may subject the user to criminal sanctions under       *
; applicable laws, as well as to civil liability for the breach of    *
; the terms and conditions of this license.                           *
;					                              *
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   *
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   *
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 	      *
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   *
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 	      *
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.	              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;								      *
;**********************************************************************
;                                                                     *
;    Notes:   This is the LCD driver for X-10 Demo Board  .	      *
;                                                                     *
;**********************************************************************

#include <p16F877.inc>
	errorlevel -302 ; supress "register not in bank0, check page bits" message

#define	LCD_DATA	PORTD		; LCD data lines interface
#define	LCD_DATA_TRIS	TRISD
#define	LCD_CTRL	PORTE		; LCD control lines interface
#define	LCD_CTRL_TRIS	TRISE

; PORTE bits
#define	LCD_E	LCD_CTRL,2		; LCD Enable control line
#define	LCD_RW	LCD_CTRL,1		; LCD Read/Write control line
#define	LCD_RS	LCD_CTRL,0		; LCD Register-Select control line

; PORTD bits
DB7		EQU	7		; LCD dataline 7 (MSB)
DB6		EQU	6		; LCD dataline 6
DB5		EQU	5		; LCD dataline 5
DB4		EQU	4		; LCD dataline 4
DB3		EQU	3		; LCD dataline 3
DB2		EQU	2		; LCD dataline 2
DB1		EQU	1		; LCD dataline 1
DB0		EQU	0		; LCD dataline 0 (LSB)

; LCD Module commands
DISP_ON		EQU	0x00C	; Display on
DISP_ON_C	EQU	0x00E	; Display on, Cursor on
DISP_ON_B	EQU	0x00F	; Display on, Cursor on, Blink cursor
DISP_OFF	EQU	0x008	; Display off
CLR_DISP	EQU	0x001	; Clear the Display
ENTRY_INC	EQU	0x006	;
ENTRY_INC_S	EQU	0x007	;
ENTRY_DEC	EQU	0x004	;
ENTRY_DEC_S	EQU	0x005	;
DD_RAM_ADDR	EQU	0x080	; Least Significant 7-bit are for address
DD_RAM_UL	EQU	0x080	; Upper Left coner of the Display

grp0	udata
LCD_TEMP	res	1		; LCD subroutines internal use
lcd_var		res	1
lcd_var2	res	1
lcd_busy_wait	res	1

	code

delay15ms macro
	movlw	.60
	movwf	lcd_var2
	movlw	.60
	movwf	lcd_var
	decfsz	lcd_var,f
	goto	$-1
	decfsz	lcd_var2,f
	goto	$-3
        nop                     ; nop for breakpoint
	endm

;*****************************************************************************
; LCD Module Subroutines
;*****************************************************************************
LCD_READ macro
	banksel	LCD_DATA_TRIS
	movlw	0xFF
	movwf	LCD_DATA_TRIS
	banksel	LCD_DATA
	endm

LCD_WRITE macro
	banksel LCD_DATA_TRIS
	clrf	LCD_DATA_TRIS
	banksel	LCD_DATA
	endm

;----------------------------------------------------------------------
; LCDINIT
; Initilize LC-Display Module
; Should be modified to your needs (i.e. display type, cursor on/off, etc.)
; OK
;----------------------------------------------------------------------
LCDINIT
	global	LCDINIT

	banksel	LCD_CTRL
				; Busy-flag is not yet valid
	clrf	LCD_CTRL	; clear control pins
	banksel	ADCON1
	movlw	0x02
	movwf	ADCON1		; set RE2-RE0 as digital I/O
	banksel	TRISE
	clrf	TRISE           ;set PORTE to output
	banksel	LCD_CTRL
	LCD_WRITE		; call macro to set port d to output	
				; Busy Flag is not valid from here
	delay15ms
	movlw	0x038		
	MOVWF   PORTD
	bsf     LCD_E
	bcf     LCD_E
	delay15ms
				; Busy Flag is not valid from here
	movlw	0x038		
	MOVWF   PORTD
	bsf     LCD_E
	bcf     LCD_E
				; Busy Flag is not valid from here
	movlw	0x038		
	MOVWF   PORTD
	bsf     LCD_E
	bcf     LCD_E
				; Busy Flag should be valid from here
	movlw	0x038		; 8-bit-interface, 2-lines
	pagesel	LCDPUTCMD
	call	LCDPUTCMD
	movlw	0x000		; disp.off, curs.off, no-blink
	pagesel	LCDDMODE
	call	LCDDMODE
	pagesel	LCDCLEAR
	call	LCDCLEAR
	movlw	0x004		; disp.on, curs.off
	pagesel	LCDDMODE
	call	LCDDMODE
	movlw	0x002		; auto-inc (shift-cursor)
	pagesel	LCDEMODE
	call	LCDEMODE
	return

;----------------------------------------------------------------------
; LCD_ENABLE
; Pulses LCD enable pin
; OK
;----------------------------------------------------------------------
lcd_enable
	BSF	LCD_E	; LCD E-line High
	bcf	LCD_E	; LCD E-line Low
	return

;----------------------------------------------------------------------
; LCDBUSY
; Returns when LCD busy-flag is inactive
; OK
;----------------------------------------------------------------------
LCDBUSY
	LCD_READ
	bcf	LCD_RS		; Set LCD for command mode
	bsf	LCD_RW		; Setup to read busy flag
	clrf	lcd_busy_wait
lcd_loopback
	incfsz	lcd_busy_wait,f	; timeout after checking 255 times.
	goto	lcd_continue
	goto	lcdnotbusy
lcd_continue
	bcf	LCD_E		; LCD E-line is low
	nop			
	nop
	bsf	LCD_E		; LCD E-line High
	nop			
	nop
	btfsc	LCD_DATA,DB7
	goto	lcd_loopback	; we are busy so do this again

lcdnotbusy
	bcf	LCD_E		; LCD E-line is low
	bcf	LCD_RW
	LCD_WRITE
	return

;----------------------------------------------------------------------
; LCDCLEAR
; Clears display and returns cursor to home position (upper-left corner).
;
;----------------------------------------------------------------------
LCDCLEAR
	global	LCDCLEAR
	movlw	0x001
	pagesel	LCDPUTCMD
	call	LCDPUTCMD
	return

;----------------------------------------------------------------------
;  L1homeLCD - Moves the cursor to home position on Line 1       
;----------------------------------------------------------------------
L1homeLCD
	global	L1homeLCD
	movlw	DD_RAM_ADDR|0x00 ; Send command to move cursor to 
	pagesel	LCDPUTCMD	
	call	LCDPUTCMD	 ; home position on line 1
	return

;----------------------------------------------------------------------
;  L2homeLCD - Moves the cursor to home position on Line 2         
;----------------------------------------------------------------------
L2homeLCD
	global	L2homeLCD
	movlw	DD_RAM_ADDR|0x28 ; Send command to move cursor to
	pagesel	LCDPUTCMD	
	call	LCDPUTCMD	 ; home position on line 2
	return

;----------------------------------------------------------------------
; LCDEMODE
; Sets entry mode of display.
; Required entry mode must be set in W
;  b0	: 0 = no display shift	1 = display shift
;  b1	: 0 = auto-decrement	1 = auto-increment
;  b2-7	: don't care
;----------------------------------------------------------------------
LCDEMODE
	global	LCDEMODE
	ANDLW	0x003		; Strip upper bits
	IORLW	0x004		; Function set
	pagesel	LCDPUTCMD
	call	LCDPUTCMD
	return

;----------------------------------------------------------------------
; LCDDMODE
; Sets display control.
; Required display mode must be set in W
;  b0	: 0 = cursor blink off	1 = cursor blink on
;  b1	: 0 = cursor off	1 = cursor on
;  b2	: 0 = display off	1 = display on (display data remains in DDRAM)
;  b3-7	: don't care
;----------------------------------------------------------------------
LCDDMODE
	global	LCDDMODE
	ANDLW	0x007		; Strip upper bits
	IORLW	0x008		; Function set
	pagesel	LCDPUTCMD
	call	LCDPUTCMD
	return

;----------------------------------------------------------------------
; LCDPUTCHAR
; Sends character to LCD
; Required character must be in W
;----------------------------------------------------------------------
LCDPUTCHAR
	global	LCDPUTCHAR
	banksel	LCD_TEMP
	movwf	LCD_TEMP	; Character to be sent is in W
	pagesel	LCDBUSY
	call	LCDBUSY		; Wait for LCD to be ready
	banksel LCD_CTRL
	bcf	LCD_RW		; Set LCD in read mode
	bsf	LCD_RS		; Set LCD in data mode
	banksel	LCD_TEMP
	movf	LCD_TEMP, W
	banksel	LCD_DATA
	movwf	LCD_DATA	; Send data to LCD
	banksel LCD_CTRL
	bsf	LCD_E		; LCD E-line High
	banksel LCD_CTRL
	bcf	LCD_E		; LCD E-line Low
	return

;----------------------------------------------------------------------
; LCDPUTCMD
; Sends command to LCD
; Required command must be in W
;
;----------------------------------------------------------------------
LCDPUTCMD
	global	LCDPUTCMD
	banksel	LCD_TEMP
	movwf	LCD_TEMP	; Command to be sent is in W
	pagesel	LCDBUSY
	call	LCDBUSY		; Wait for LCD to be ready
	banksel LCD_CTRL
	bcf	LCD_RW		; Set LCD in read mode
	bcf	LCD_RS		; Set LCD in command mode
	banksel	LCD_TEMP
	movf	LCD_TEMP, W
	banksel	LCD_DATA
	movwf	LCD_DATA	; Send data to LCD
 	banksel LCD_CTRL
	bsf	LCD_E		; LCD E-line High
	banksel LCD_CTRL
	bcf	LCD_E		; LCD E-line Low
	return

	end                     ; directive 'end of module'

