;**********************************************************************
;                                                                     *
;    Filename:	    x10demo.asm					      *
;    Date:          10/15/02                                          *
;    File Version:   1.00                                             *
;                                                                     *
;    Author:        Jon Burroughs                                     *
;    Company:       Microchip Technology                              *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                    Software License Agreement                       *
;                                                                     *
; The software supplied herewith by Microchip Technology              *
; Incorporated (the "Company") for its PICmicro Microcontroller is   *
; intended and supplied to you, the Companys customer, for use       *
; solely and exclusively on Microchip PICmicro Microcontroller        *
; products. The software is owned by the Company and/or its           *
; supplier, and is protected under applicable copyright laws. All     *
; rights are reserved. Any use in violation of the foregoing          *
; restrictions may subject the user to criminal sanctions under       *
; applicable laws, as well as to civil liability for the breach of    *
; the terms and conditions of this license.                           *
;					                              *
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   *
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   *
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 	      *
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   *
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 	      *
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.	              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;           16F877.lkr   or   16F877a.lkr                             *
;           p16F877.inc  or   p16F877a.inc                            *
;	    x10lib.asm					              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:   Test file for X-10 functionality.  	              *
;                                                                     *
;**********************************************************************

	list      p=16F877		; list directive to define processor
        #include <p16F877.inc>		; processor specific variable definitions
	#include "x10lib.inc"		; X10 constant definitions	
        errorlevel -302			; suppress "register not in bank0, check page bits" message.	

;**********************************************************************
;   Variables                                                         *
;**********************************************************************
				;address 0x70 is reserved for compatibility with ICD
_WREG		EQU	0x71    ; used to save WREG during interrupt
_STATUS         EQU     0x72    ; used to save STATUS during interrupt
_PCLATH		EQU	0x73	; used to save PCLATH during interrupt
_FSR		EQU	0x74	; used to save FSR during interrupt

Hours		EQU	0x75
Mins		EQU	0x76
Secs		EQU	0x77
Ticks		EQU	0x78	;Tick is 25ms	

OTHER_VARS	udata	0x20

TempHouse	res	1
TempUnit	res	1

T1SimCount	res	1

_TempHours	res	1
_TempMins	res	1
_TempUnit	res	1

Dimmer		res	1

#define	  TRIAC  TRISA, 5 	;clear bit to turn on triac (PORTA bit 5 must be set)

;**********************************************************************
;   External declarations                                             *
;**********************************************************************
	extern	ZeroCrossing, InitX10Tx, DecodeHouseOrUnit, EndX10Envelope, InitX10Variables
	extern	X10Flags, TxHouse, TxUnit, TxFunction, RxHouse, RxUnit, RxFunction	;global variables

;----------------------------------------------------------------------
; Startup Code
;----------------------------------------------------------------------
STARTUP	code

  	nop		        ; 0x0000  (This nop necessary for ICD operation)
  	goto 	Start		; 0x0001
  	nop			; 0x0002
  	nop			; 0x0003
;	goto	ISR		; 0x0004  (Go to Interrupt Service Routine)

;****************************************************************************************************
ISRcode	code	0x0005
;****************************************************************************************************		
;----------------------------------------------------------------------
; ISR- 
;	Interrupt Service Routine.  Determine cause of interrupt and perform appropriate action.
;----------------------------------------------------------------------
;	Interrupt Sources:
;	TMR0:		Times duration of 1ms 120kHz burst and onset of 2nd & 3rd phase bursts.
;	TMR1: 		Time-keeping clock, also generates 1 interrupt every 25ms for keyscan.	
;	RBO/INT:	INTF is set upon each zero-crossing, 
;			used to synchronize transmission and
;			reception of X-10 signals (120kHz bursts).
;	TMR2:		Used with CCP1 to generate 120kHz PWM output,
;			Also used with postscaler to generate dimmer timing increments
;			of .125us for 32 interrupts/half-cycle.		
;
;----------------------------------------------------------------------
ISR
	;PUSH
	movwf	_WREG		; Save WREG
	movf	STATUS,W	; Save STATUS
	clrf	STATUS
	movwf	_STATUS
	movf	PCLATH, W
	movwf	_PCLATH
	movf	FSR, W
	movwf	_FSR
	
	pagesel	ZeroCrossing
	btfsc	INTCON, INTF	;External interrupt?
	call	ZeroCrossing	;Yes, then handle ZeroCrossing	

	pagesel	EndX10Envelope
	btfsc	INTCON, T0IF	;Timer 0 interrupt?
	call	EndX10Envelope	;Yes, then EndEnvelope

	bsf	INTCON, GIE	;Re-enable global interrupts

	;POP
	clrf    STATUS          ; select bank 0
	movf    _FSR, W		; restore the FSR
	movwf   FSR
	movf	_PCLATH, W
	movwf	PCLATH
	movf   _STATUS, W	;Swap original status register value into W
	movwf   STATUS		;Restore status register from W register
	swapf   _WREG, F	;Swap _WREG nibbles and return to _WREG
	swapf   _WREG, W	;Swap _WREG to W to restore original W value
				; without affecting STATUS.
	retfie

;****************************************************************************************************
PROG1	code
;****************************************************************************************************
Start
	banksel	TRISA
	bcf	TRISA, 1		;make LED1 pin an ouput

	pagesel	InitTRIAC
	call	InitTRIAC
	pagesel	InitTimer0
	call	InitTimer0
	pagesel	InitTimer1		
	call	InitTimer1
	pagesel	InitPWM
	call	InitPWM
	pagesel	InitZeroXDetect
	call	InitZeroXDetect
	pagesel	InitX10Variables
	call	InitX10Variables
	pagesel	EnableInterrupts
	call	EnableInterrupts

;****************************************************************************************************
;****************************************************************************************************
; Main Loop Code
;	-is just an endless loop
;****************************************************************************************************
;****************************************************************************************************
Main
	SkipIfTxDone
	goto	$-1
	SendX10Address HouseA, Unit1

	SkipIfTxDone
	goto	$-1
	SendX10Command HouseA, On

	SkipIfTxDone
	goto	$-1
	SendX10Command HouseA, Off

	SkipIfTxDone
	goto	$-1
	SendX10Command HouseA, AllLightsOn

	SkipIfTxDone
	goto	$-1
	SendX10Command HouseA, AllUnitsOff

	SkipIfTxDone
	goto	$-1
	SendX10Command HouseA, AllLightsOn

	SkipIfTxDone
	goto	$-1
	SendX10Command HouseA, AllUnitsOff

;	goto	$
	goto	Main

;****************************************************************************************************
;INITIALIZATION ROUTINES
;****************************************************************************************************	
;----------------------------------------------------------------------
; InitTimer1-                                   
;----------------------------------------------------------------------
;Timer1 is used with an external 32kHz crystal oscillator and configured
;to generate an interrupt 1/25ms.
;	TMR1 interrupt= 1/32kHz * prescaler * (FFFFh+1-preload)
;			1/32kHz * 1         * (FFFFh+1-FCE0h) = 25msec
;	- Initialize Timer1 with 8000h             
;		- Configure Timer1                         
;		- Prescale 1:1                         
;		- Enable Oscillator                    
;		- DO NOT SYNCHRONIZE                   
;		- External Clock Source                
;		- Enable Timer1                        
;   - Clear Timer1 Overflow Interrupt Flag     
;   - Enable Timer1 Overflow Interrupt                       
;----------------------------------------------------------------------
InitTimer1
	BANKSEL TMR1H		;Initialize Timer1 with FCE0h
	movlw	0xFC		
	movwf	TMR1H
	movlw	0xE0			
	movwf	TMR1L		
	movlw	B'00001111'	;Configure Timer1 
	movwf	T1CON		
	bcf	PIR1,TMR1IF	;Clear Timer1 Overflow Interrupt Flag
	BANKSEL PIE1
	bsf	PIE1,TMR1IE	;Enable Timer1 Overflow Interrupt
	return

;----------------------------------------------------------------------
; InitTimer0
;----------------------------------------------------------------------	
; Timer0 is used to time the duration of the 1ms 120kHz burst and the 
;	onset of phase2 and phase3 bursts if implemented.
;	Fosc = 7.6800MHz, TMR0 clock source is instruction cycle clock, Prescaler = 64 so:
;		- 1ms delay with TMR0 preload of 0xE2 (.226) 
;		- 1.7667ms delay with TMR0 preload of 0xCB (.203)                                   
;----------------------------------------------------------------------
InitTimer0
	banksel	OPTION_REG
	movlw	b'10010101'	;no RBPU, internal instruction cycle clock, 1:64 prescaler
	movwf	OPTION_REG	
	bcf	INTCON, T0IF	;Clear Timer1 Overflow Interrupt Flag
;	bsf	INTCON, T0IE	;Enable Timer0 Overflow Interrupt
	return


;----------------------------------------------------------------------
; InitZeroXDetect                                 
;----------------------------------------------------------------------
InitZeroXDetect
	banksel	OPTION_REG
	bsf	OPTION_REG, INTEDG	;interrupt on rising edge	
	bsf	INTCON, INTE		;Enable INT External interrupt on RB0	
	return


;----------------------------------------------------------------------
; InitTRIAC                              
;----------------------------------------------------------------------
InitTRIAC
	banksel	PORTA
	bcf	TRIAC		;set TRIAC pin low
	banksel	TRISA
	bcf	TRISA, 5	;make TRIAC pin an output
	return


;----------------------------------------------------------------------
;InitPWM
;	- Output on RC2 is initially disabled
;	- Initialize PWM to produce 120kHz oscillation for X-10 transmitter
;	- Main oscillator is 7.680MHz, Tosc = 1/7.680MHz
;	- PWM period = 1/120kHz =  8.333us
;	- PWM period = [(PR2) + 1] * 4 * Tosc * (TMR2 prescale value)
;		     = [(0) + 1] * 4 * 1/7.680MHz * (16) = 8.333us
;	- PR2 = 0, TMR2 prescaler = 16
;	- 50% PWM duty cycle = 8.333us/2 = DCxB9:DCxB0 * Tosc * (TMR2 prescale value) 
;	                   DCxB9:DCxB0 = 8.333us/2/Tosc/(TMR2 prescale value) = 2 
;	- postscaler = 15 generates TMR2 interrupt every 15 * 8.333us= .125ms
;		for approximately 32 interrupts/half-cycle for dimmer control.
;----------------------------------------------------------------------
InitPWM
	banksel TRISC
	bsf	XOUT		;disable output

	banksel	CCP1CON
	clrf	CCP1CON		;CCP module is off
	banksel	TMR2
	clrf	TMR2		;Clear Timer2

	banksel	PR2
	clrf	PR2		;set the PWM period to 1/120kHZ by setting PR2 to zero
	banksel	CCPR1L
	clrf	CCPR1L		;set duty cycle to 50% by writing 2 = b'0000000010' to DCxB9:DCxB0 		
	clrf	INTCON

	banksel	PIE1
	clrf	PIE1
	banksel	PIR1
	clrf	PIR1
	movlw	b'00101100'
	banksel	CCP1CON	
	movwf	CCP1CON		;configure the CCP1 module for PWM operation

	movlw	b'00000110'
	banksel	T2CON	
	movwf	T2CON		;set the TMR2 prescaler to 16, postscaler to 15, and enable TMR2	

	return


;----------------------------------------------------------------------
; InitVariables                              
;----------------------------------------------------------------------
InitVariables 

	banksel	Secs
	clrf	Secs		;initialize clock variables
	clrf	Mins

	movlw	.12
	movwf	Hours

	return

;----------------------------------------------------------------------
; EnableInterrupts
;	- First, clear peripheral interrupt flags            
;   	- Enable Peripheral Interrupts             
;   	- Enable Global Interrupts                    
;----------------------------------------------------------------------
EnableInterrupts 
	BANKSEL	PIR1
	clrf	PIR1
	clrf	PIR2
	
	bsf	INTCON, PEIE	;Enable Peripheral Interrupts
	bsf	INTCON, GIE	;Enable Global Interrupts
	return	


;***********************
;END OF PROGRAM
;***********************

	end			;directive 'end of program'       
