;**********************************************************************
;                                                                     *
;    Filename:	    x10lib.inc					      *
;    Date:          10/15/02                                          *
;    File Version:   1.00                                             *
;                                                                     *
;    Author:        Jon Burroughs                                     *
;    Company:       Microchip Technology                              *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                    Software License Agreement                       *
;                                                                     *
; The software supplied herewith by Microchip Technology              *
; Incorporated (the "Company") for its PICmicro Microcontroller is   *
; intended and supplied to you, the Companys customer, for use       *
; solely and exclusively on Microchip PICmicro Microcontroller        *
; products. The software is owned by the Company and/or its           *
; supplier, and is protected under applicable copyright laws. All     *
; rights are reserved. Any use in violation of the foregoing          *
; restrictions may subject the user to criminal sanctions under       *
; applicable laws, as well as to civil liability for the breach of    *
; the terms and conditions of this license.                           *
;					                              *
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   *
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   *
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 	      *
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,   *
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 	      *
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.	              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;           p16F877.inc  or   p16F877a.inc                            *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:   This file provides constant definitions for X10 calls.  *
;**********************************************************************

	list      p=16F877	      ; list directive to define processor
        #include <p16F877.inc>        ; processor specific variable definitions
        errorlevel -302               ; suppress "register not in bank0, check page bits" message.	

;**********************************************************************
;   X-10 board signal/pin definitions                                 *
;**********************************************************************    

;#define	ZERO_X PORTB, 0
#define	XOUT   TRISC, 2	;clear to enable 120kHz output, set disable 120kHz output
#define	XIN    PORTC, 3

#define	LED1   PORTA, 1
#define	LED2   PORTA, 2


;**********************************************************************
;   X10 function definitions                                          *
;**********************************************************************
#define	AllUnitsOff	b'01010101'	; 'All units off'			b'0000'
#define	AllLightsOn	b'01010110'	; 'All lights on'			b'0001'		
#define	On		b'01011001'	; 'On'					b'0010'	
#define Off		b'01011010'	; 'Off'					b'0011'	
#define Dim		b'01100101'	; 'Dim'					b'0100'	
#define Bright		b'01100110'	; 'Bright'				b'0101'	
#define	AllLightsOff	b'01101001'	; 'All lights off'			b'0110'
#define	ExtendedCode	b'01101010'	; 'Extended Code'			b'0111'
#define	HailRequest	b'10010101'	; 'Hail Request'			b'1000'		
#define	HailAcknowledge	b'10010110'	; 'Hail Acknowledge'			b'1001'
#define	PresetDim	b'10011001'	; 'Pre-set Dim'(actually, 101x)		b'1010'	
#define	ExtendedData	b'10100101'	; 'Extended Data (Analog)'		b'1100'
#define	StatusOn	b'10100110'	; 'Status = ON'				b'1101'
#define	StatusOff	b'10101001'	; 'Status = OFF'			b'1110'	
#define StatusRequest	b'10101010'	; 'Status request'			b'1111'

#define	HouseA		0x00
#define	HouseB		0x01
#define	HouseC		0x02
#define	HouseD		0x03
#define	HouseE		0x04
#define	HouseF		0x05
#define	HouseG		0x06
#define	HouseH		0x07
#define	HouseI		0x08
#define	HouseJ		0x09
#define	HouseK		0x0A
#define	HouseL		0x0B
#define	HouseM		0x0C
#define	HouseN		0x0D
#define	HouseO		0x0E	
#define	HouseP		0x0F

#define	Unit1		0x00
#define	Unit2		0x01
#define	Unit3		0x02
#define	Unit4		0x03
#define	Unit5		0x04
#define	Unit6		0x05
#define	Unit7		0x06
#define	Unit8		0x07
#define	Unit9		0x08
#define	Unit10		0x09
#define	Unit11		0x0A
#define	Unit12		0x0B
#define	Unit13		0x0C
#define	Unit14		0x0D
#define	Unit15		0x0E	
#define	Unit16		0x0F

;**********************************************************************
;   X10 flag definitions                                              *
;**********************************************************************
#define		RxBit			X10Flags, 0
#define		TxBit			X10Flags, 1
#define		RxCommandFlag		X10Flags, 2	;Set when command received, clear when address received
#define		TxCommandFlag		X10Flags, 3	;Set when sending command, clear when sending address
#define		X10RxFlag		X10Flags, 4	;Set when receive buffer ready, cleared when receive done
#define		X10TxFlag		X10Flags, 5	;Set to transmit, gets cleared when transmit complete
#define		SecondTxFlag		X10Flags, 6	;Set to indicate 2nd transmission
#define		WaitFlag		X10Flags, 7	;Set to wait for 6 cycles, clear when done


;****************************************************************************************************
;Define X10 Library Macros
;****************************************************************************************************
;----------------------------------------------------------------------
; SkipIfTxDone
;----------------------------------------------------------------------
SkipIfTxDone	macro
	banksel	X10Flags
	btfsc	X10TxFlag	;If X10TxFlag is clear, then last transmit is done
	endm

;----------------------------------------------------------------------
; SkipIfRxDone
;----------------------------------------------------------------------
SkipIfRxDone	macro
	banksel	X10Flags
	btfss	X10RxFlag	;If X10RxFlag is set, then data has been received
	endm

;----------------------------------------------------------------------
; SkipIfWaitDone
;----------------------------------------------------------------------
SkipIfWaitDone	macro
	banksel	X10Flags
	btfss	WaitFlag	;If X10RxFlag is clear, then wait is complete
	endm


;----------------------------------------------------------------------
; SendX10Address (HouseConstant, UnitConstant)
;----------------------------------------------------------------------
SendX10Address	macro	House, Unit
	movlw	House		;Copy house argument into TxHouse
	banksel	TxHouse
	movwf	TxHouse
	movlw	Unit		;Copy unit argument into TxUnit
	banksel	TxUnit
	movwf	TxUnit
	banksel	X10Flags
	bcf	TxCommandFlag	;Clear the TxCommandFlag for sending address
	pagesel	InitX10Tx
	call	InitX10Tx
	bsf	X10TxFlag	;Set the X10TxFlag to begin transmission

	endm	

;----------------------------------------------------------------------
; SendX10AddressVar
;----------------------------------------------------------------------
SendX10AddressVar	macro
	banksel	X10Flags
	bcf	TxCommandFlag	;Clear the TxCommandFlag for sending address
	pagesel	InitX10Tx
	call	InitX10Tx
	bsf	X10TxFlag	;Set the X10TxFlag to begin transmission

	endm	


;----------------------------------------------------------------------
; SendX10Command (House, Function)
;----------------------------------------------------------------------
SendX10Command	macro	House, Function
	banksel	House
	movlw	House		;Copy house argument into TxHouse
	banksel	TxHouse
	movwf	TxHouse
	movlw	Function	;Copy function argument into TxFunc
	banksel	TxFunction
	movwf	TxFunction
	banksel	X10Flags
	bsf	TxCommandFlag	;Set the TxCommandFlag for sending command
	pagesel	InitX10Tx
	call	InitX10Tx
	bsf	X10TxFlag	;Set the X10TxFlag to begin transmission

	endm

;----------------------------------------------------------------------
; SendX10CommandVar
;----------------------------------------------------------------------
SendX10CommandVar	macro
	banksel	X10Flags
	bsf	TxCommandFlag	;Set the TxCommandFlag for sending command
	pagesel	InitX10Tx
	call	InitX10Tx
	bsf	X10TxFlag	;Set the X10TxFlag to begin transmission

	endm


;----------------------------------------------------------------------
; ReceiveX10Message
;	-may be expanded to receive extended data
;----------------------------------------------------------------------
ReceiveX10Message	macro
	movf	RxHouse2, W		;Convert received house code to 0-15
	pagesel	DecodeHouseOrUnit
	call	DecodeHouseOrUnit
	movwf	RxHouse
	
	pagesel	ReceiveUnit
	btfss	RxCommandFlag		;Was a command received?
	goto	ReceiveUnit		;   No, then convert key code to unit address
	movf	RxKey2, W		;   Yes, then key code is function code
	movwf	RxFunction
	pagesel	EndReceiveX10Message
	goto	EndReceiveX10Message

ReceiveUnit
	movf	RxKey2, W		;Convert received key code to 0-15
	pagesel	DecodeHouseOrUnit
	call	DecodeHouseOrUnit
	movwf	RxUnit

EndReceiveX10Message
	bcf	X10RxFlag		
	endm

;----------------------------------------------------------------------
; SkipIfReceiveReady
;----------------------------------------------------------------------
SkipIfReceiveReady	macro
	banksel	X10Flags
	btfss	X10RxFlag	;If X10RxFlag is set, then receive buffer is ready
	endm

;----------------------------------------------------------------------
; SkipIfCommandRcvd
;----------------------------------------------------------------------
SkipIfCommandRcvd	macro
	banksel	X10Flags
	btfss	TxCommandFlag	;If TxCommandFlag is set, then command has been received
	endm

;----------------------------------------------------------------------
; SkipIfAddressRcvd
;----------------------------------------------------------------------
SkipIfAddressRcvd	macro
	banksel	X10Flags
	btfsc	TxCommandFlag	;If TxCommandFlag is clear, then address has been received
	endm


;----------------------------------------------------------------------
; Wait
;----------------------------------------------------------------------
Wait	macro
	movlw	.6
	movwf	Count
	bsf	WaitFlag		

	endm


